﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using UnityEngine.UI;

public class DialogueMaster : MonoBehaviour {
	public const int MAX_MSG = 100;
	public const int SHOW_TIME = 100;
    public player p;

	bool showing = false;
	public DialogueMessage[] messages = new DialogueMessage[MAX_MSG];
	public int message_count = 0;
    public int start = 0;
	public int counter = SHOW_TIME;
    Text text;
    Image portrait;
    AudioSource audioSource;
    GameObject option1;
    GameObject option2;
    GameObject cursor1;
    Image box;
    GameObject cursor2;

	void Start () {
        portrait = ((Image)this.GetComponent<Transform>().FindChild("Portrait").GetComponent<Image>());
        text = this.GetComponentInChildren<Text>();
        audioSource = this.GetComponent<AudioSource>();
        option1 = this.GetComponent<Transform>().FindChild("Option 1").gameObject;
        option2 = this.GetComponent<Transform>().FindChild("Option 2").gameObject;
        cursor1 = this.GetComponent<Transform>().FindChild("Cursor 1").gameObject;
        cursor2 = this.GetComponent<Transform>().FindChild("Cursor 2").gameObject;
        box = ((Image)this.GetComponent<Transform>().FindChild("Box").GetComponent<Image>());
    }
	
	// Update is called once per frame
	void Update () {
        if(counter == -1)
        {
            return;
        }
		else if (counter > 0) {
			counter --;
		}
		else if (message_count - start > 0) {
            p.isDialogueMode = true;
			showing = true;
            show();
            int cur_ind = Mathf.FloorToInt((start) % MAX_MSG);
            if (messages[cur_ind] is ConvoMessage)
            {
                DisplayText(messages[cur_ind]);
                //start++;
                counter = -1;
            }
            else
            {
                DisplayText(messages[cur_ind]);
                start++;
                counter = SHOW_TIME;
            }
		} else if (showing == true) {
            audioSource.Stop();
			showing = false;
            p.isDialogueMode = false;
			hide ();
		}

	}

    public void chooseOption(int opt)
    {
        int cur_ind = Mathf.FloorToInt((start) % MAX_MSG);
        ConvoMessage mes = ((ConvoMessage)messages[cur_ind]);
        if (opt == 1)
        {
            this.DisplayText(new DialogueMessage(mes.response1, mes.picLoc));
        }
        else
        {
            this.DisplayText(new DialogueMessage(mes.response2, mes.picLoc));
        }
        option1.SetActive(false);
        option2.SetActive(false);
        cursor1.SetActive(false);
        cursor2.SetActive(false);
        start++;
        counter = SHOW_TIME;
    }

	public void SendMessage(string mesInp, Sprite picInp)
	{
		if (message_count - start < MAX_MSG) {
			messages[Mathf.FloorToInt(message_count % MAX_MSG)] = new DialogueMessage (mesInp, picInp, p.voice);
            message_count++;
        }
    }
    public void SendMessage(DialogueMessage inp)
    {
        if (message_count - start < MAX_MSG)
        {
            messages[Mathf.FloorToInt(message_count % MAX_MSG)] = inp;
            message_count++;
        }
    }
    public void SendMessage(string[] messages, bool[] isPlayer, Sprite picInp, AudioClip aud = null)
    {
        for(int i = 0; i < messages.Length; i ++)
        {
            if(isPlayer[i])
            {
                SendMessage(new DialogueMessage(messages[i]));
            }
            else
            {

                SendMessage(new DialogueMessage(messages[i], picInp, aud));
            }
        }
    }
    public void skipMessage()
    {
        this.counter = 1;
    }

    private void DisplayText(DialogueMessage inp)
    {
        if(inp is SoundMessage)
        {
            audioSource.clip = ((SoundMessage)inp).aud;
            audioSource.Play();
            return;
        }
        if(inp is DestroyMessage)
        {
            ((DestroyMessage)inp).sendingObject.SetActive(false);
            p.TriggerList.Remove(((DestroyMessage)inp).sendingObject.GetComponent<Collider2D>());
            return;
        }

        #region itemMessage handling
        if(inp is ItemMessage)
        {
            ItemMessage inpItem = (ItemMessage)inp;
            p.Inventory.Add(new Item(inpItem.itemID, inpItem.itemimg));
            box.sprite = inpItem.picLoc;
            text.text = inpItem.message;
            return;

        }
        #endregion

        #region audio handling
        if (inp.audio != null && !inp.isPlayer)
        {
            if (audioSource.clip != inp.audio || !audioSource.isPlaying)
            {
                audioSource.clip = inp.audio;
                audioSource.Play();
            }
        }
        else if (inp.isPlayer)
        {
            if (audioSource.clip != p.voice || !audioSource.isPlaying)
            {
                audioSource.clip = p.voice;
                audioSource.Play();
            }
        }
        else { audioSource.Stop(); }
        #endregion

        #region portrait handling
        if(inp.isPlayer)
        {
            box.sprite = p.dialogueImage;
            portrait.enabled = false;
        }
        else
        {
            box.sprite = inp.picLoc;

            //portrait.enabled = true;
        }
        #endregion
        
        if(inp is ConvoMessage)
        {
            portrait.enabled = false;
            this.GetComponentInChildren<Text>().text = inp.message;
            option1.SetActive(true);
            option1.GetComponent<Text>().text = ((ConvoMessage)inp).option1;
            cursor1.SetActive(true);
            option2.SetActive(true);
            option2.GetComponent<Text>().text = ((ConvoMessage)inp).option2;
            p.setConvoMode(true, cursor1, cursor2);
        }
        else
        {
            text.text = inp.message;
        }
	}

	public void hide()
	{
		this.transform.localScale = new Vector3 (0, 0, 0);
	}
    public void clearQ()
    {
        this.start = this.message_count;
        counter = 0;
    }

	private void show()
	{
		this.transform.localScale = new Vector3 (1, 1, 1);
	}
}
